
import java.io.*;
import java.util.*;

//import org.apache.xerces.dom.TextImpl;
//import org.apache.xerces.parsers.DOMParser;

import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import javax.xml.parsers.*;

//wddx
import com.allaire.wddx.WddxSerializer;

//cfx
import com.allaire.cfx.*;


public class CFX_XMLParser
    implements CustomTag
{
    public static final String ATTRIBUTE_LIST = "ATTRIBUTE_LIST";
    public static final String TEXT_ELEMENT = "INNER_TEXT";
    public static final String CHILD_LIST = "CHILD_LIST";
    public static final String CDATA_ELEMENT = "CDATA_ELEMENT";
    public static final String LIST_DELIM = ",";
	public static final String DOCUMENT_BUILDER_FACTORY = "org.apache.xerces.jaxp.DocumentBuilderFactoryImpl";
    
    /** parses a string of XML into a WDDX structure */
    public static String parse(String xml, String factoryName)
        throws Exception
    {
        DocumentBuilder builder = ((DocumentBuilderFactory)Class.forName(factoryName).newInstance()).newDocumentBuilder();
        InputSource xmlsource = new InputSource(new StringReader(xml));
		
		
        Document doc = builder.parse(xmlsource);

        Hashtable root = new Hashtable(20);
        root = walk(doc, root);
        WddxSerializer ws = new WddxSerializer();

        // Create a writer to store the generated WDDX
        StringWriter sw = new StringWriter();
        
        // Serialize the data
        ws.serialize(root, sw);
        
        // Return the WDDX packet
        return sw.toString();
        
        
        
    } //end parse
    
    /** recursivly walks a DOM tree and populates a nested hash table. */
    protected static Hashtable walk(Node node, Hashtable hash)
        throws Exception
    {
        
        // is there anything to do?
        if (node == null) {
            return hash;
        }
        int type = node.getNodeType();
        
        switch (type) {
        // print document
            case Node.DOCUMENT_NODE: {
                walk(((Document)node).getDocumentElement(), hash);
                break;
            }

            // print element with attributes
            case Node.ELEMENT_NODE: {
                
                Hashtable elemHash = new Hashtable(node.getChildNodes().getLength());
                //eliminate duplicates
                String hkey = node.getNodeName();
                int hnum = 2;
                while(hash.containsKey(hkey)) {
                    hkey = node.getNodeName() + hnum;
                    hnum++;
                }
                
                hash.put(hkey, elemHash);
                NamedNodeMap attrs = node.getAttributes();
                if (attrs != null && attrs.getLength() > 0) {
                    StringBuffer attrlist = new StringBuffer(attrs.getLength());
                    for (int i=0;i<attrs.getLength();i++) {
                        attrlist.append(attrs.item(i).getNodeName());
                        attrlist.append(CFX_XMLParser.LIST_DELIM);
                        elemHash.put(attrs.item(i).getNodeName(), 
                            attrs.item(i).getNodeValue());
                    }
                    //trim traling ,
                   
                    attrlist.deleteCharAt(attrlist.length()-1);
                    elemHash.put(CFX_XMLParser.ATTRIBUTE_LIST, attrlist.toString());
                }
                NodeList children = node.getChildNodes();
                
                if (children != null) {
                    int len = children.getLength();
                    if (len > 0) {
                        StringBuffer childlist = new StringBuffer();
                        for (int i = 0; i < len; i++) {
                            if (children.item(i).getNodeType() == Node.ELEMENT_NODE) {
                                
                                String key = children.item(i).getNodeName();
                                int num = 2;
                                while(elemHash.containsKey(key)) {
                                    key = children.item(i).getNodeName() + num;
                                    num++;
                                }
                                childlist.append(key);
                                childlist.append(CFX_XMLParser.LIST_DELIM);
                            }
                            elemHash = walk(children.item(i), elemHash);
                        }
                        if (childlist.length() > 0) {
                            childlist.deleteCharAt(childlist.length()-1);
                        }
                        elemHash.put(CFX_XMLParser.CHILD_LIST, childlist.toString());
                    }
                }
                break;
            }

            // handle entity reference nodes
            case Node.ENTITY_REFERENCE_NODE: {
                NodeList children = node.getChildNodes();
                if (children != null) {
                    int len = children.getLength();
                    for (int i = 0; i < len; i++) {
                        walk(children.item(i), hash);
                    }
                }
                break;
            }

            // print text
            case Node.CDATA_SECTION_NODE: {
                hash.put(CFX_XMLParser.CDATA_ELEMENT, node.getNodeValue());
                break;
            }
            case Node.TEXT_NODE: {
                hash.put(CFX_XMLParser.TEXT_ELEMENT, node.getNodeValue());
                break;
            }
        }
        return hash;
    } //end walk
    
    public void processRequest(com.allaire.cfx.Request request,
        com.allaire.cfx.Response response) 
            throws java.lang.Exception
    {
        String var = "wddxOutput";
		String factoryName = DOCUMENT_BUILDER_FACTORY;
        if (!request.attributeExists("XML")) 
        {
            throw new Exception("CFX_XMLParser: Required Attribute XML not found.");
		}
        if (request.attributeExists("OUTPUT"))
        {
            var = request.getAttribute("OUTPUT");
	}
	if (request.attributeExists("FACTORYNAME")) {
		factoryName = request.getAttribute("FACTORYNAME");
	}
	response.setVariable(var, CFX_XMLParser.parse(request.getAttribute("XML"), factoryName));
    }
    
    
    public static void main(String[] args) 
        throws Exception
    {
        String xml = "<?xml version='1.0'?><root><sub>inner text</sub></root>";
        System.out.println("\n\n"+ CFX_XMLParser.parse(xml, DOCUMENT_BUILDER_FACTORY));
    }
}

